/******************************************************************************
 *
 * 
 *
 *
 * Copyright (C) 1997-2007 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#ifndef _GLE_H
#define _GLE_H

#include "qtbc.h"
#include <qlist.h>
#include <qdict.h>
#include "sortdict.h"
#include "graphoutputformat.h"
class ClassDef;
class FileDef;
class QTextStream;
class GleNodeList;
class ClassSDict;
class MemberDef;
class Definition;
class DirDef;
class GroupDef;
class GleGroupCollaboration;



/** @brief A node in a gle graph */
class GleNode
{
  public:
    enum GraphType { Dependency, Inheritance, Collaboration, Hierarchy, CallGraph };
    enum TruncState { Unknown, Truncated, Untruncated };
    GleNode(int n,const char *lab,const char *tip,const char *url,
            bool rootNode=FALSE,ClassDef *cd=0);
   ~GleNode();
    void addChild(GleNode *n,
                  int edgeColor=EdgeInfo::Purple,
                  int edgeStyle=EdgeInfo::Solid,
                  const char *edgeLab=0,
                  const char *edgeURL=0,
                  int edgeLabCol=-1
                 );
    void addParent(GleNode *n);
    void deleteNode(GleNodeList &deletedList,SDict<GleNode> *skipNodes=0);
    void removeChild(GleNode *n);
    void removeParent(GleNode *n);
    int findParent( GleNode *n );
    void write(QTextStream &t,GraphType gt,GraphOutputFormat f,
               bool topDown,bool toChildren,bool backArrows,bool reNumber);
    int  m_subgraphId;
    void clearWriteFlag();
    void writeXML(QTextStream &t,bool isClassGraph);
    void writeDEF(QTextStream &t);
    QCString label() const { return m_label; }
    int  number() const { return m_number; }
    bool isVisible() const { return m_visible; }
    TruncState isTruncated() const { return m_truncated; }

  private:
    void colorConnectedNodes(int curColor);
    void writeBox(QTextStream &t,GraphType gt,GraphOutputFormat f,
                  bool hasNonReachableChildren, bool reNumber=FALSE);
    void writeArrow(QTextStream &t,GraphType gt,GraphOutputFormat f,GleNode *cn,
                    EdgeInfo *ei,bool topDown, bool pointBack=TRUE, bool reNumber=FALSE);
    const GleNode   *findDocNode() const; // only works for acyclic graphs!
    void markAsVisible(bool b=TRUE) { m_visible=b; }
    void markAsTruncated(bool b=TRUE) { m_truncated=b ? Truncated : Untruncated; }
    int              m_number;
    QCString         m_label;     //!< label text
    QCString         m_tooltip;   //!< node's tooltip
    QCString         m_url;       //!< url of the node (format: remote$local)
    QList<GleNode>  *m_parents;   //!< list of parent nodes (incoming arrows)
    QList<GleNode>  *m_children;  //!< list of child nodes (outgoing arrows)
    QList<EdgeInfo> *m_edgeInfo;  //!< edge info for each child
    bool             m_deleted;   //!< used to mark a node as deleted
    bool             m_written;   //!< used to mark a node as written
    bool             m_hasDoc;    //!< used to mark a node as documented
    bool             m_isRoot;    //!< indicates if this is a root node
    ClassDef *       m_classDef;  //!< class representing this node (can be 0)
    bool             m_visible;   //!< is the node visible in the output
    TruncState       m_truncated; //!< does the node have non-visible children/parents

    friend class GleGfxHierarchyTable;
    friend class GleClassGraph;
    friend class GleInclDepGraph;
    friend class GleNodeList;
    friend class GleCallGraph;
    friend class GleGroupCollaboration;

    friend QCString computeMd5Signature(
                      GleNode *root, GraphType gt,
                      GraphOutputFormat f, 
                      bool lrRank, bool renderParents,
                      bool backArrows,
                      QCString &graphStr
                     );
};

inline int GleNode::findParent( GleNode *n )
{
    if( !m_parents )
        return -1;
    return m_parents->find(n);
}

/** @brief Represents a graphical class hierarchy */
class GleGfxHierarchyTable
{
  public:
    GleGfxHierarchyTable();
   ~GleGfxHierarchyTable();
    void writeGraph(QTextStream &t,const char *path);
  
  private:
    void addHierarchy(GleNode *n,ClassDef *cd,bool hide);
    void addClassList(ClassSDict *cl);

    QList<GleNode> *m_rootNodes; 
    QDict<GleNode> *m_usedNodes; 
    static int      m_curNodeNumber;
    GleNodeList    *m_rootSubgraphs;
};

/** @brief Representation of a class inheritance or dependency graph */
class GleClassGraph
{
  public:
    GleClassGraph(ClassDef *cd,GleNode::GraphType t);
   ~GleClassGraph();
    bool isTrivial() const;
    QCString writeGraph(QTextStream &t,GraphOutputFormat f,const char *path,
                    const char *relPath, bool TBRank=TRUE,bool imageMap=TRUE);

    void writeXML(QTextStream &t);
    void writeDEF(QTextStream &t);
    QCString diskName() const;

  private:
    void buildGraph(ClassDef *cd,GleNode *n,bool base);
    void determineVisibleNodes(QList<GleNode> &queue,int &maxNodes,bool includeParents);
    void determineTruncatedNodes(QList<GleNode> &queue,bool includeParents);
    void addClass(ClassDef *cd,GleNode *n,int prot,const char *label,
                  const char *usedName,const char *templSpec,
                  bool base);

    GleNode        *   m_startNode;
    QDict<GleNode> *   m_usedNodes;
    static int         m_curNodeNumber;
    GleNode::GraphType m_graphType;
    QCString           m_diskName;
};

/** @brief Representation of an include dependency graph */
class GleInclDepGraph
{
  public:
    GleInclDepGraph(FileDef *fd,bool inverse);
   ~GleInclDepGraph();
    QCString writeGraph(QTextStream &t, GraphOutputFormat f,const char *path,
                    const char *relPath,
                    bool writeImageMap=TRUE);
    bool isTrivial() const;
    QCString diskName() const;
    void writeXML(QTextStream &t);

  private:
    void buildGraph(GleNode *n,FileDef *fd);
    void determineVisibleNodes(QList<GleNode> &queue,int &maxNodes);
    void determineTruncatedNodes(QList<GleNode> &queue);

    GleNode        *m_startNode;
    QDict<GleNode> *m_usedNodes;
    static int      m_curNodeNumber;
    QCString        m_diskName;
    int             m_maxDistance;
    bool            m_inverse;
};

/** @brief Representation of an call graph */
class GleCallGraph
{
  public:
    GleCallGraph(MemberDef *md,bool inverse);
   ~GleCallGraph();
    QCString writeGraph(QTextStream &t, GraphOutputFormat f,
                        const char *path,const char *relPath,bool writeImageMap=TRUE);
    void buildGraph(GleNode *n,MemberDef *md);
    bool isTrivial() const;
    void determineVisibleNodes(QList<GleNode> &queue, int &maxNodes);
    void determineTruncatedNodes(QList<GleNode> &queue);
    
  private:
    GleNode        *m_startNode;
    static int      m_curNodeNumber;
    QDict<GleNode> *m_usedNodes;
    int             m_maxDistance;
    int             m_recDepth;
    bool            m_inverse;
    QCString        m_diskName;
    Definition *    m_scope;
};

/** @brief Representation of an directory dependency graph */
class GleDirDeps
{
  public:
    GleDirDeps(DirDef *dir);
   ~GleDirDeps();
    bool isTrivial() const;
    QCString writeGraph(QTextStream &out,
                        GraphOutputFormat format,
                        const char *path,
                        const char *relPath,
                        bool writeImageMap=TRUE);
  private:
    DirDef *m_dir;
};

/** @brief Representation of a group collaboration graph */
class GleGroupCollaboration
{
  public :
    enum EdgeType 
    {  tmember = 0,
       tclass,
       tnamespace,
       tfile,
       tpages,
       tdir,
       thierarchy 
    };

    class Link
    {
      public:
        Link(const QCString lab,const QCString &u) : label(lab), url(u) {}
        QCString label;
        QCString url;
    };

    class Edge
    {
      public :
        Edge(GleNode *start,GleNode *end,EdgeType type) 
          : pNStart(start), pNEnd(end), eType(type)
        { links.setAutoDelete(TRUE); }

        GleNode* pNStart;
        GleNode* pNEnd;
        EdgeType eType;

        QList<Link> links;
        void write( QTextStream &t, int& curNodeId );
    };

    GleGroupCollaboration(GroupDef* gd);
    ~GleGroupCollaboration();
    QCString writeGraph(QTextStream &t, GraphOutputFormat format,
                    const char *path,const char *relPath,
                    bool writeImageMap=TRUE);
    void buildGraph(GroupDef* gd);
    bool isTrivial() const;
  private :
    void addCollaborationMember( Definition* def, QCString& url, EdgeType eType );
    void addMemberList( class MemberList* ml );
    void writeGraphHeader(QTextStream &t);
    Edge* addEdge( GleNode* _pNStart, GleNode* _pNEnd, EdgeType _eType,
                   const QCString& _label, const QCString& _url );

    GleNode        *m_rootNode;
    int             m_curNodeId;
    QDict<GleNode> *m_usedNodes;
    QCString        m_diskName;
    QList<Edge>     m_edges;
};

/** @brief Helper class to run gle from doxygen.
 */
class GleRunner
{
  public:
    /** Creates a runner for a gle \a file. */
    GleRunner(const char *file);

    /** Adds an additional job to the run.
     *  Performing multiple jobs one file can be faster.
     */
    void addJob(const char *format,const char *output);

    /** Runs gle for all jobs added. */
    bool run();
  private:
    QList<QCString> m_jobs;
    QCString m_file;
};


/** Generated a graphs legend page */
void generateGraphLegend(const char *path);

void writeGleGraphFromFile(const char *inFile,const char *outDir,
                           const char *outFile,GraphOutputFormat format);
QString getGleImageMapFromFile(const QString& inFile, const QString& outDir,
                               const QCString& relPath);


#endif
